/* Measures VSWR.  ???? ADC connected to LPT1
 *	pin ?:	?
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dos.h>
#include <conio.h>
#include "bigchar.h"

#define	VREF		5.00	/* Reference voltage		*/
#define	FORWCHAN	0	/* Forward input channel	*/
#define	REVCHAN		1	/* Reverse input channel	*/

static int read_adc(int);

int lpt_base = 0x378;	/* LPT base address	*/
int lpt_irq = 7;	/* Not used		*/
int clkdelay = 1000;
int debug = 0;
unsigned char databits;
unsigned char controlbits;
int oldadc0 = -1;
int oldadc1 = -1;

main(int argc, char *argv)
{
	double	vf,vr,swr;
	int adc0,adc1;
	char os[128];

	clrscr();
	gotoxy(35,5);
	printf("VSWR meter");
	while (!kbhit()) {
		gotoxy(1,24);
		clreol();
		printf("Press a key to exit");
		adc0 = read_adc(FORWCHAN);
		adc1 = read_adc(REVCHAN);
		/* Prevent flickering screen */
		if (oldadc0 == adc0 && oldadc1 == adc1) {
			delay(200);
			continue;
		}
		oldadc0 = adc0;
		oldadc1 = adc1;
		vf = (double)adc0 * VREF / 255.0;
		vr = (double)adc1 * VREF / 255.0;
		if (adc0 > 0 ) {
			if (adc1 != adc0) {
				swr = (vf + vr) / (vf - vr);
				sprintf(os,"%.2f",swr);
			} else {
				sprintf(os,"\x1e\x1f\x1e\x1f");
			}
		} else {
			sprintf(os,"----");
		}
		clear_lines(10,15);
		print_bigtext((80-strlen(os)*6)/2,10,os);
		sprintf(os,"Vforward = %.2f Vreverse = %.2f",vf,vr);
		gotoxy((80 - strlen(os))/2,19);
		clreol();
		printf(os);
		delay(200);
	}
	getch();
	gotoxy(1,24);
	return 0;
}

/* Low level routines */

void
init_port(void) {
	databits = 0;
	controlbits = 0x02;
	outportb(lpt_base,databits);
	outportb(lpt_base+2,controlbits);
}

void
setcs(int c)
{
	if (c != 0)
		controlbits &= 0xf7;
	else
		controlbits |= 0x8;
	outportb(lpt_base+2,controlbits);
}

void
setclk(int c)
{
	if (c != 0)
		controlbits &= 0xfd;
	else
		controlbits |= 0x2;
	outportb(lpt_base+2,controlbits);
}

void
setdi(int c)
{
	if (c != 0)
		controlbits |= 0x4;
	else
		controlbits &= 0xfb;
	outportb(lpt_base+2,controlbits);
}

int
getdo(void)
{
	unsigned char statusbits;

	statusbits = inportb(lpt_base+1);
	return((statusbits & 8) == 8);
}

int
getsars(void)
{
	unsigned char statusbits;

	statusbits = inportb(lpt_base+1);
	return((statusbits & 32) == 32);
}

void
clock_once(void)
{
	long i;

	setclk(1);
	for(i = 0L ; i < clkdelay ; i++)
		;
	setclk(0);
	for(i = 0L ; i < clkdelay ; i++)
		;
	if (debug > 0)
		printf("status=0x%02x do=%d sars=%d\n",
				inportb(lpt_base+1),getdo(),getsars());
}

void
sendbit(int b)
{
	setdi(b);
	clock_once();
}

/* Read ADC channel */
int
read_adc(int c)
{
	int res,clks;
	long i;

	setclk(0);
	setcs(0);

	/* Start bit */
	sendbit(1);

	/* Single / ~DIF bit */
	sendbit(1);

	/* Channel bits */
	sendbit(c & 1);
	sendbit((c & 2) >> 1);

	/* Read value */
	res = 0;
	for(clks = 0 ; clks < 8; clks++) {
		clock_once();
		res = (res << 1) + getdo();
	}
	if (debug > 0)
		printf("res=0x%02x\n",res);

	/* All done */
	setcs(1);

	return (res);
}

